package yrpg.fi.cas.plugin.operate;

import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.plugin.AbstractOperationServicePlugIn;
import kd.bos.entity.plugin.PreparePropertysEventArgs;
import kd.bos.entity.plugin.args.AfterOperationArgs;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.sdk.plugin.Plugin;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import yrpg.fi.cas.plugin.other.webservice.OutServiceLocator;
import yrpg.fi.cas.plugin.other.webservice.OutServicePortType;

import java.math.BigDecimal;
import java.security.MessageDigest;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 单据操作插件  审核后推送付款处理单至保融
 */
public class PushCaspayBillToBaoRongAfterAuditOpPlugin extends AbstractOperationServicePlugIn implements Plugin {

    public void onPreparePropertys(PreparePropertysEventArgs e){
        e.getFieldKeys().add("billstatus");
        e.getFieldKeys().add("yrpg_paystatus");
        e.getFieldKeys().add("yrpg_pushfundpay");
        e.getFieldKeys().add("id");
        e.getFieldKeys().add("billno");
        e.getFieldKeys().add("openorg");
        e.getFieldKeys().add("applyorg");
        e.getFieldKeys().add("expectdate");
        e.getFieldKeys().add("sourcebillnumber");
        e.getFieldKeys().add("yrpg_transactiontype");
        e.getFieldKeys().add("payeebank");
        e.getFieldKeys().add("yrpg_brcode");
        e.getFieldKeys().add("settletype");
        e.getFieldKeys().add("urgentflag");
        e.getFieldKeys().add("payeracctbank");
        e.getFieldKeys().add("actpayamt");
        e.getFieldKeys().add("payeename");
        e.getFieldKeys().add("payeebanknum");
        e.getFieldKeys().add("ispersonpay");
        e.getFieldKeys().add("yrpg_fundnumber");
        e.getFieldKeys().add("yrpg_pushfailurecaus");
    }

    @Override
    public void afterExecuteOperationTransaction(AfterOperationArgs e) {
        super.afterExecuteOperationTransaction(e);

        DynamicObject[] bills = e.getDataEntities();

        for(DynamicObject bill:bills){
            //单据状态
            String billstatus=bill.getString("billstatus");

            //支付状态
            String paystatus=bill.getString("yrpg_paystatus");

            //是否推送保融支付
            Boolean pushfundpay=bill.getBoolean("yrpg_pushfundpay");

            //付款处理单据状态为“已审核”、支付状态为“1-未推送或6-推送失败或者8-重付待推”且是送否推送保融支付为是的单据
            if("C".equals(billstatus)&&("1".equals(paystatus)||"6".equals(paystatus)||"8".equals(paystatus))&&pushfundpay){
                this.pushCaspayBillToBaoRong(bill);
            }
        }
    }

    public void pushCaspayBillToBaoRong(DynamicObject caspaybill){

        //盐值  salt
        String salt=System.getProperty("yrpg.fi.cas.baorongzj.salt");

        String url=System.getProperty("yrpg.fi.cas.baorongzj.url");

        //对接系统代码   srcoutsystemcode
        String srcoutsystemcode=System.getProperty("yrpg.fi.cas.baorongzj.srcoutsystemcode");

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        Date now=new Date();

        long caspayid = caspaybill.getLong("id");//付款处理单id
        String billno = caspaybill.getString("billno");//付款处理单编码
        DynamicObject openorg=caspaybill.getDynamicObject("openorg");//核算组织
        String openorgno ="";//核算组织编码
        if(openorg!=null){
            openorgno=openorg.getString("yrpg_companynum");
        }
        DynamicObject applyorg=caspaybill.getDynamicObject("applyorg");//申请组织
        String applyorgno ="";
        if(applyorg!=null){
            applyorgno=applyorg.getString("yrpg_companynum");
        }
        Date expectdate = caspaybill.getDate("expectdate");//期望付款日期
        String sourcebillno=caspaybill.getString("sourcebillnumber");//源单编码
        DynamicObject transactiontype=caspaybill.getDynamicObject("yrpg_transactiontype");//交易类型
        String transactiontypeno = "";//交易类型编码
        if(transactiontype!=null){
            transactiontypeno=transactiontype.getString("number");
        }

        //收款银行编码
        String payeebankno="";

        //收款银行
        DynamicObject payeebank=caspaybill.getDynamicObject("payeebank");
        if(payeebank!=null){
            payeebankno=payeebank.getString("number");
        }

        //版本号
        int brcode=caspaybill.getInt("yrpg_brcode");

        //结算方式
        String brbillingmethodcode="";//结算方式代码
        DynamicObject settletype=caspaybill.getDynamicObject("settletype");
        if(settletype!=null){
            brbillingmethodcode=settletype.getString("yrpg_settlementtypcode");
        }

        Boolean urgentflag = caspaybill.getBoolean("urgentflag");//加急支付
        int urgentfl=0;
        if(urgentflag){
            urgentfl=1;
        }

        DynamicObject payeracctbank=caspaybill.getDynamicObject("payeracctbank");//银行账号
        String payeracctbankno = "";
        if(payeracctbank!=null){
            payeracctbankno=payeracctbank.getString("bankaccountnumber");
        }

        String currencyno ="CNY";//币别
        BigDecimal actpayamt = caspaybill.getBigDecimal("actpayamt");//付款金额
        String recaccbankname = caspaybill.getString("payeename");//收款人实名
        String payeebanknum = caspaybill.getString("payeebanknum");//收款人账号
        Boolean ispersonpay = caspaybill.getBoolean("ispersonpay");//对私支付 true：对私  false:对公
        int ispersonp=2;
        if(ispersonpay){
            ispersonp=1;
        }

        String detail ="<detail>"+
                "<SRCSERIALNO>"+caspayid+"</SRCSERIALNO>"+
                "<SRCNOTECODE>"+billno+"</SRCNOTECODE>"+
                "<ORGCODE>"+openorgno+"</ORGCODE>"+
                "<APPLYORGCODE>"+applyorgno+"</APPLYORGCODE>"+
                "<PAYDATE>"+sdf.format(expectdate)+"</PAYDATE>"+
                "<PAYTYPECODE>"+transactiontypeno+"</PAYTYPECODE>"+
                "<SETTLEMENTMODECODE>"+brbillingmethodcode+"</SETTLEMENTMODECODE>"+
                "<OPPBANKLOCATIONCODE>"+payeebankno+"</OPPBANKLOCATIONCODE>"+
                "<ISURGENT>"+urgentfl+"</ISURGENT>"+
                "<OURORGCODE>"+applyorgno+"</OURORGCODE>"+
                "<OURBANKACCOUNTNUMBER>"+payeracctbankno+"</OURBANKACCOUNTNUMBER>"+
                "<OURCURCODE>"+currencyno+"</OURCURCODE>"+
                "<OURAMOUNT>"+actpayamt+"</OURAMOUNT>"+
                "<OPPOBJECTNAME>"+recaccbankname+"</OPPOBJECTNAME>"+
                "<OPPBANKACCOUNTNUMBER>"+payeebanknum+"</OPPBANKACCOUNTNUMBER>"+
                "<OPPBANKACCOUNTNAME>"+recaccbankname+"</OPPBANKACCOUNTNAME>"+
                "<OPPDIRECTCURCODE>"+currencyno+"</OPPDIRECTCURCODE>"+
                "<OPPPRIVATEFLAG>"+ispersonp+"</OPPPRIVATEFLAG>"+
                "<EXTFIELD1>"+srcoutsystemcode+"</EXTFIELD1>"+
                "<EXTFIELD2>"+sourcebillno+"</EXTFIELD2>"+
                "</detail>";

        //req参数
        String req="<req><head><ALLAMOUNT>"+actpayamt+"</ALLAMOUNT><ALLCOUNT>"+1+"</ALLCOUNT></head>"+
                "<list>"+detail.toString()+"</list></req>";

        //MD5值
        String md5str=md5String(req,salt);

        //pub参数
        String pub="<pub>" +
                "<SRCOUTSYSTEMCODE>"+srcoutsystemcode+"</SRCOUTSYSTEMCODE>" +
                "<SRCBATCHNO>"+caspayid+"-"+brcode+"</SRCBATCHNO> " +
                "<TRANSCODE>PMSQ01</TRANSCODE>" +
                "<TRANSDATETIME>"+sdf.format(now)+"</TRANSDATETIME>" +
                "<MD5>"+md5str+"</MD5>" +
                "</pub>";


        //入参
        String xml="<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
                "<MBS>"+pub+req+"</MBS>";

        Map<String, Object> logMap = new HashMap<>();
        logMap.put("opname", "推送付款处理到资金接口");
        logMap.put("thirdappname", "资金接口");
        logMap.put("opdesc",url);
        logMap.put("request", xml);
        logMap.put("status", "true");
        logMap.put("username", "yy");

        try {
            OutServiceLocator locatar = new OutServiceLocator();
            OutServicePortType outServiceHttpPort = locatar.getOutServiceHttpPort();
            String response =outServiceHttpPort.outSystemWS(xml);

            Document document = DocumentHelper.parseText(response);
            Element rootElement = document.getRootElement();
            Element pubele = rootElement.element("pub");

            //返回代码
            Element rescodeele=pubele.element("RESPCODE");
            String rescode=(String)rescodeele.getData();

            //失败信息
            Element respmsgele=pubele.element("RESPMSG");
            String respmsg=(String)respmsgele.getData();

            if("0".equals(rescode)){//"0"指令接收成功

                Element respele = rootElement.element("resp");
                Element listele=respele.element("list");
                Element detailele=listele.element("detail");
                Element notecode=detailele.element("NOTECODE");
                String nocode=(String)notecode.getData();//资金系统单据号

                if(nocode!=null&&!"".equals(nocode)){
                    caspaybill.set("billstatus","E");//单据状态 付款处理中
                    caspaybill.set("yrpg_paystatus","4");//支付状态 支付中
                    caspaybill.set("yrpg_fundnumber",nocode);//资金系统单号
                    //清空失败原因
                    caspaybill.set("yrpg_pushfailurecaus","");//失败原因

                    SaveServiceHelper.update(caspaybill);//更新付款处理单

                    logMap.put("response", "推送付款处理单成功！");
                    saveApiLog(logMap);
                }else{
                    caspaybill.set("billstatus","C");//单据状态 已审核
                    caspaybill.set("yrpg_paystatus","6");//支付状态  推送失败

                    Element payinfo=detailele.element("PAYINFO");
                    String payinfostr=(String)payinfo.getData();//失败信息

                    caspaybill.set("yrpg_pushfailurecaus",payinfostr);//失败原因
                    brcode++;
                    caspaybill.set("yrpg_brcode",brcode);

                    SaveServiceHelper.update(caspaybill);//更新付款处理单

                    logMap.put("status", "false");
                    logMap.put("response", "推送付款处理失败！错误信息是"+payinfostr);

                    saveApiLog(logMap);
                }
            }else{//失败
                caspaybill.set("billstatus","C");//单据状态 已审核
                caspaybill.set("yrpg_paystatus","6");//支付状态  推送失败
                caspaybill.set("yrpg_pushfailurecaus",respmsg);//失败原因
                brcode++;
                caspaybill.set("yrpg_brcode",brcode);

                SaveServiceHelper.update(caspaybill);//更新付款处理单

                logMap.put("status", "false");
                logMap.put("response", "推送付款处理失败！错误信息是"+respmsg);

                saveApiLog(logMap);
            }

        } catch (Exception e) {
            logMap.put("status", "false");
            logMap.put("response", "推送付款处理失败！错误信息是"+e.getMessage());
            saveApiLog(logMap);
            throw new RuntimeException(e);
        }
    }

    /**
     * 对字符串进行md5加密
     * @param md5String
     * @param md5Key
     * @return md5加密后的密文
     */
    private static String md5String(String md5String,String md5Key){
        String md5Value = MD5Encoder(md5String + md5Key);
        return md5Value;
    }

    public final static String MD5Encoder(String s) {
        return MD5Encoder(s,"utf-8");
    }

    public final static String MD5Encoder(String s, String charset) {
        try {
            byte[] btInput = s.getBytes(charset);
            MessageDigest mdInst = MessageDigest.getInstance("MD5");
            mdInst.update(btInput);
            byte[] md = mdInst.digest();
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < md.length; i++) {
                int val = ((int) md[i]) & 0xff;
                if (val < 16){
                    sb.append("0");
                }
                sb.append(Integer.toHexString(val));
            }
            return sb.toString();
        } catch (Exception e) {
            return null;
        }
    }

    public static void saveApiLog(Map<String,Object> logMap){
        DynamicObject apiLog = BusinessDataServiceHelper.newDynamicObject("openapi_log_data");
        //api编号
        apiLog.set("opname",logMap.get("opname"));
        //URL
        apiLog.set("opdesc",logMap.get("opdesc"));
        //请求参数
        if(logMap.get("request")!=null){
            apiLog.set("request",logMap.get("request").toString().length()>2000?logMap.get("request").toString().substring(0, 2000):logMap.get("request"));
        }else {
            apiLog.set("request","");
        }

        if (logMap.get("response")!=null){
            //响应结果
            apiLog.set("response",logMap.get("response").toString().length()>2000?logMap.get("response").toString().substring(0, 2000):logMap.get("response"));
        }else {
            apiLog.set("response","");
        }
        //调用时间
        apiLog.set("opdate",new Date());
        //调用者
        apiLog.set("username",logMap.get("username"));
        //第三方应用
        apiLog.set("thirdappname",logMap.get("thirdappname"));
        //调用状态
        apiLog.set("status",logMap.get("status"));
        //保存日志
        SaveServiceHelper.save(new DynamicObject[]{apiLog});

    }
}